import { useState, useEffect, useMemo } from 'react';
import { Input, Pagination, Empty } from 'antd';
import { api, RouterInputs } from "@nice/client";
import { LetterCard } from "../LetterCard";
import { NonVoid } from "@nice/utils";
import { SearchOutlined } from '@ant-design/icons';
import debounce from 'lodash/debounce';
import { postDetailSelect } from '@nice/common';
export default function LetterList({ params }: { params: NonVoid<RouterInputs["post"]["findManyWithPagination"]> }) {
    const [searchText, setSearchText] = useState('');
    const [currentPage, setCurrentPage] = useState(1);

    const { data, isLoading } = api.post.findManyWithPagination.useQuery({
        page: currentPage,
        pageSize: params.pageSize,
        where: {
            OR: [{
                title: {
                    contains: searchText
                }
            }],
            ...params?.where
        },
        select: {
            ...postDetailSelect,
            ...params.select
        }
    });
    useEffect(() => {
        console.log(data)
    }, [data])
    // Debounced search function
    const debouncedSearch = useMemo(
        () =>
            debounce((value: string) => {
                setSearchText(value);
                setCurrentPage(1);
            }, 300),
        []
    );
    // Cleanup debounce on unmount
    useEffect(() => {
        return () => {
            debouncedSearch.cancel();
        };
    }, [debouncedSearch]);
    const handleSearch = (value: string) => {
        debouncedSearch(value);
    };

    const handlePageChange = (page: number) => {
        setCurrentPage(page);
        // Scroll to top when page changes
        window.scrollTo({ top: 0, behavior: 'smooth' });
    };

    return (
        <div className="flex flex-col h-full">
            {/* Search Bar */}
            <div className="p-6 mb-6 transition-all ">
                <div className="max-w-2xl ">
                    <Input
                        placeholder="搜索信件标题..."
                        allowClear
                        size="large"
                        onChange={(e) => handleSearch(e.target.value)}
                        prefix={<SearchOutlined className="text-gray-400" />}
                    />
                </div>
            </div>

            {/* Content Area */}
            <div className="flex-grow overflow-auto px-4">
                {isLoading ? (
                    <div className="grid grid-cols-1 md:grid-cols-2 lg:grid-cols-3 gap-6 animate-pulse">
                        {[...Array(6)].map((_, index) => (
                            <div key={index} className="h-48 bg-gray-100 rounded-lg"></div>
                        ))}
                    </div>
                ) : data?.items.length ? (
                    <>
                        <div className="grid grid-cols-1 md:grid-cols-1 lg:grid-cols-2 gap-6 mb-6">
                            {data.items.map((letter: any) => (
                                <LetterCard key={letter.id} letter={letter} />
                            ))}
                        </div>
                        <div className="flex justify-center pb-6">
                            <Pagination
                                current={currentPage}
                                total={data.totalCount}
                                pageSize={params.pageSize}
                                onChange={handlePageChange}
                                showSizeChanger={false}
                                showQuickJumper
                            />
                        </div>
                    </>
                ) : (
                    <div className="flex flex-col justify-center items-center h-96">
                        <Empty
                            image={Empty.PRESENTED_IMAGE_SIMPLE}
                            description={
                                <span className="text-gray-600">
                                    {searchText ? "未找到相关信件" : "暂无信件"}
                                </span>
                            }

                            className="flex flex-col items-center"
                        />
                    </div>
                )}
            </div>


        </div>
    );
}
